from General                              import findFormContainer
from Globals                              import DTMLFile
from Products.OrderedFolder.OrderedFolder import OrderedFolder
from OFS.Folder                           import Folder
from OFS.DTMLMethod                       import DTMLMethod

manage_addSmartSectionForm = DTMLFile('dtml/addSmartSection', globals())

def manage_addSmartSection(self, id, title='', completeSetup=0, REQUEST=None):
    ''' '''
    section = SmartSection()
    section.id = str(id)
    section.title = title
    self._setObject(id, section)

    # Okay, I hope many of the usability issues will go away now. Using this setup, the
    # Smart Section should work out of the box. 
    if completeSetup:
        # First, set the required 'isFormContainer' variable
        if not self.hasProperty('isFormContainer'):
            self.aq_parent.aq_parent._setProperty('isFormContainer', 1, type='boolean')

        # Second, try to add the transparent folder 'Forms', where all the element forms
        # will be stored 
        try:
            from Products.TransparentFolders import TransparentFolder
            TransparentFolder.manage_addTransparentFolder(self, 'Forms', 'All Element Forms are stored here')
        except ImportError:
            pass

        # Third, create a default viewing method, so that the user can start using Smart Sections
        # right away
        try:
            self._setObject('index_html', DTMLMethod(_default_index_html %id, __name__ = 'index_html'))
        except:
            self._setObject('render', DTMLMethod(_default_index_html %id, __name__ = 'render'))
                     

    manage_addNewItem = DTMLMethod(_manage_addNewItem, __name__ = 'manage_addNewItem')
    root = findFormContainer(self.aq_parent.aq_parent)
    try:
        root._setObject('manage_addNewItem', manage_addNewItem)
    except:
        pass

    if REQUEST:
        return self.manage_main(self, REQUEST)



class SmartSection(OrderedFolder):
    ''' '''

    _is_transparent = 0

    meta_type = 'Smart Section'
    icon = 'misc_/SmartSections/Section'

    _meta_types = ()


    def all_meta_types(self):
        ''' '''
        return self._meta_types


    def manage_move_object_to_top(self, id, nextURL='', REQUEST=None):
        ''' '''
        self.move_object_to_top(id)

        if nextURL:
            return REQUEST.RESPONSE.redirect(nextURL)
        elif REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_up(self, id, nextURL='', REQUEST=None):
        ''' '''
        self.move_object_up(id)

        if nextURL:
            return REQUEST.RESPONSE.redirect(nextURL)
        elif REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_down(self, id, nextURL='', REQUEST=None):
        ''' '''
        self.move_object_down(id)

        if nextURL:
            return REQUEST.RESPONSE.redirect(nextURL)
        elif REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_move_object_to_bottom(self, id, nextURL='', REQUEST=None):
        ''' '''
        self.move_object_to_bottom(id)

        if nextURL:
            return REQUEST.RESPONSE.redirect(nextURL)
        elif REQUEST:
            return self.manage_main(self, REQUEST)


    def manage_delete_objects(self, ids=[], nextURL='', REQUEST=None):
        ''' '''
        self.manage_delObjects(ids)

        if nextURL:
            return REQUEST.RESPONSE.redirect(nextURL)
        elif REQUEST:
            return self.manage_main(self, REQUEST)


    def getEditHTML(self, object):
        ''' '''
        dict = {'objectId'    : object.getId(),
                'objectClass' : object.__class__.__name__,
                'sectionURL'  : self.absolute_url(),
                'nextURL'     : self.REQUEST['URL'],
                'position'    : self.get_object_position(object.getId())}
        html =      _editHTML %dict

        if self.get_object_position(object.getId()) != 0:
            html = html + _upHTML %dict

        if self.get_object_position(object.getId()) != len(self._objects)-1:
            html = html + _downHTML %dict

        return html


    def render(self, mode='view'):
        ''' '''
        html = ''
        for object in self.objectValues():
            if mode == 'edit':
                html = html + self.getEditHTML(object) + '\n'
            html = html + object(mode) + '<br>\n'

        if mode == 'edit':
            dict = {'nextURL'     : self.REQUEST['URL'],
                    'endPosition' : len(self.objectValues()),
                    'sectionURL'  : self.absolute_url()}
            html = html + _newHTML %dict 
    
        return html
    

    def content(self, mode='view'):
        ''' '''
        if self.REQUEST and self.REQUEST.has_key('mode'):
            mode = self.REQUEST['mode']
        return self.render(mode)


    def __call__(self, mode='view'):
        ''' '''
        return self.render(mode)


    def __repr__(self):
        ''' '''
        return self.render()



def initializeElement(ElementClass, constructors=()):
    ''' '''
    dict = { 'name'   : ElementClass.meta_type,
             'action' : constructors[0].__name__ }

    for constructor in constructors:
        setattr(SmartSection, constructor.__name__, constructor)


    SmartSection._meta_types = SmartSection._meta_types + (dict,)


_newHTML = '''
 <hr width="90%%" align="left" size="1" noshade>
 <a href="%(sectionURL)s/manage_addNewItem?position=%(endPosition)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/New" border="0" alt="New"></a>
'''

_editHTML = '''
 <hr width="90%%" align="left" size="1" noshade>
 <a href="%(sectionURL)s/manage_addNewItem?position=%(position)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/New" border="0" alt="New"></a>
 <a href="%(sectionURL)s/%(objectId)s/edit_%(objectClass)sForm/renderMethod?mode=edit&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Edit" border="0" alt="Edit"></a> 
 <a href="%(sectionURL)s/manage_delete_objects?ids=%(objectId)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Delete" border="0" alt="Delete"></a> 
'''

_upHTML = '''
 <a href="%(sectionURL)s/manage_move_object_to_top?id=%(objectId)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Top" border="0" alt="Top"></a> 
 <a href="%(sectionURL)s/manage_move_object_up?id=%(objectId)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Up" border="0" alt="Up"></a>
'''

_downHTML = '''
 <a href="%(sectionURL)s/manage_move_object_down?id=%(objectId)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Down" border="0" alt="Down"></a> 
 <a href="%(sectionURL)s/manage_move_object_to_bottom?id=%(objectId)s&nextURL=%(nextURL)s"
   ><img src="/misc_/SmartSections/Bottom" border="0" alt="Bottom"></a> 
<br>
'''

_manage_addNewItem = '''
<dtml-var standard_html_header>
  <br />
    <dtml-if filtered_meta_types>
      <div class="form-element">
        <form name="addForm" action="&dtml-URL1;/" method="post">
          <dtml-if "REQUEST.has_key('nextURL')">
            <input type="hidden" name="nextURL" value="&dtml-nextURL;">    
          </dtml-if>
          <dtml-if "REQUEST.has_key('position')">
            <input type="hidden" name="position" value="&dtml-position;">    
          </dtml-if>
          <dtml-if "_.len(filtered_meta_types) > 1">
            <select class="form-element" name=":action" 
              onChange="javascript:document.forms[0].submit();">
              <option value="manage_workspace" disabled>Select type to add...</option>
              <dtml-in filtered_meta_types mapping sort=name>
                <option value="&dtml.url_quote-action;">&dtml-name;</option>
              </dtml-in>
            </select>
            <input class="form-element" type="submit" name="action" value=" Add " />
          <dtml-else>
            <dtml-in filtered_meta_types mapping sort=name>
              <input type="hidden" name=":method" value="&dtml.url_quote-action;" />
              <input class="form-element" type="submit" name="submit" value=" Add &dtml-name;" />
            </dtml-in>
          </dtml-if>
        </form>
      </div>
    </dtml-if>
<dtml-var standard_html_footer>
'''


_default_index_html = '''
<dtml-var standard_html_header>

  <dtml-if "REQUEST.has_key('mode') and mode == 'edit'">
    <a href="&dtml-URL;?mode=view">Change to view</a>
  <dtml-else>
    <dtml-call "REQUEST.set('mode', 'view')">
    <a href="&dtml-URL;?mode=edit">Change to edit</a>
  </dtml-if>

  <br><hr width="100%%"><br>

  <dtml-call "RESPONSE.setCookie('mode', mode)">

  <dtml-var "%s.render(mode)">

<dtml-var standard_html_footer>
'''
